<?php
/* $Id: audioblog_client.inc,v 1.1 2005/10/28 15:26:29 leoburd Exp $ */

/**
 * @file
 * Basic functionality required to process audioblogs on the Asterisk server side.
 *
 * At the top level, this files provides functions to quickly store audioblog entries
 * in a database so that they can be later processed and uploaded to a Drupal server.
 */


// -----------------------------------------------------------------------------
// include required files
// -----------------------------------------------------------------------------

foreach (array( 'audioblog.ini', 'common.inc', 'xmlrpc.inc') as $file) {
   require_once($file);
}


// -----------------------------------------------------------------------------
// constants
// -----------------------------------------------------------------------------

	// Status of a blog entry

	define( 'AB_TO_BE_UPLOADED', 'to be uploaded');
	define( 'AB_UPLOADED', 'uploaded');
	define( 'AB_ERROR', 'error');


	// Log file

	define( 'AB_LOG_FILE', '/var/log/audioblog.log');


// -----------------------------------------------------------------------------
// global variables
// -----------------------------------------------------------------------------

	global $audioblog_db_options;
	
	global $audioblog_agi_debug_on;

// -----------------------------------------------------------------------------
// initialization
// -----------------------------------------------------------------------------


	// time limit for the upload transactions
	set_time_limit( 60);


// -----------------------------------------------------------------------------
// public function definitions
// -----------------------------------------------------------------------------

/*
 * Add a new audioblog entry to the queue so that it can be later uploaded to the server.
 * 
 * @param $entry
 *   array with all the parameters required to create a new audioblog
 *   entry (server, title, body, etc.)
 * @return
 *   boolean TRUE indicates success.  If FALSE, check $audioblog_errormsg for explanation.
 */
function audioblog_create_entry($entry) {

	global $audioblog_db_options;


	// open connection to database server
	$db_link = mysql_connect($audioblog_db_options['server'], $audioblog_db_options['user'], $audioblog_db_options['password']);

	if (!$db_link) {
		audioblog_error('Could not connect to database server: ' . mysql_error());
		return FALSE;
	}

	// select database to be used

	$db = mysql_select_db( $audioblog_db_options['db_name'], $db_link);

	if (!$db_link) {
		audioblog_error('Can\'t use ' . $audioblog_db_options['db_name'] . ' : ' . mysql_error());
		return FALSE;
	}

	
	// insert audioblog entry in the database

	$status = AB_TO_BE_UPLOADED;
	$notes = 'empty note';

	$query = "INSERT INTO audioblog_queue
	          ( title, body, file_name, file_path, file_url_tmp, xmlrpc_url, drupal_uid, phone_pin, blog_id, publish, date_created, status, notes)
	           VALUES( '%s', '%s', '%s', '%s', '%s', '%s', '%d', '%s', '%s', %d, %d, '%s', '%s')";
	$query = format_query($query,
		                    $entry['title'],
				                $entry['body'], 
		                    $entry['file_name'],
		                    $entry['file_path'],
				                $entry['file_url_tmp'],
		                    $entry['xmlrpc_url'],
		                    $entry['drupal_uid'],
		                    $entry['phone_pin'],
		                    $entry['blog_id'],
		                    $entry['publish'],
		                    $entry['date_created'],
		                    $status, 
				                $notes);

	$db_result = mysql_query($query, $db_link);

	if (!$db_result) {
		audioblog_error( 'Could not add audioblog entry to database: ' . mysql_error());
		return FALSE;
	}

	return TRUE;

}


/*
 * Upload audioblog entries stored in the queue.
 * 
 * @param $maxcount
 *   integer the maximum number of entries to process per call
 * @return
 *   boolean TRUE indicates success.  If FALSE, check audioblog_error_msg() for explanation.
 */
function upload_audioblog_entries_from_queue( $maxcount=1) {
	$uploaded_entries_count = 0;

	global $audioblog_db_options;

	// open connection to database server

	$db_link = mysql_connect($audioblog_db_options['server'], $audioblog_db_options['user'], $audioblog_db_options['password']);

	if (!$db_link) {
		audioblog_error('Could not connect to database server: ' . mysql_error());
		return -1;
	}

	// select database to be used

	$db = mysql_select_db($audioblog_db_options['db_name'], $db_link);
	
	if (!$db) {
		audioblog_error('Can\'t use ' . $audioblog_db_options['db_name'] . ': ' . mysql_error());
		return -1;
	}

	
	//
	// process the audioblog entries from the queue
	//


	// select a batch of entries waiting to be processed

	$query = "SELECT entry_id FROM audioblog_queue WHERE status = '%s' ORDER BY entry_id ASC LIMIT %d";
	$query = format_query($query, AB_TO_BE_UPLOADED, $maxcount);

	$db_result = mysql_query($query, $db_link);

	if (!$db_result ) {
		
		audioblog_error( 'Could not query audioblog entries to be uploaded: ' . mysql_error());
		return -1;
	}


	// if there are entries waiting to be uploaded...

	// ... annotate their ids...

	$entry_ids = NULL;

	while ($entry = mysql_fetch_assoc($db_result)) {
		$entry_ids[] = $entry['entry_id'];
	}


	// ...and attempt to "lock" unprocessed entries so that they are not sent off the queue more than once

	if (is_array($entry_ids)) {

		$query = ("UPDATE audioblog_queue SET status = '%s' WHERE entry_id IN (%s)");
		
		$uniqueid = mt_rand();   // generate a unique id to mark the entries being processed 
					 // by the current function call

		$entry_ids = implode(', ', $entry_ids);

		$query = format_query($query, $uniqueid, $entry_ids);

		$db_result = mysql_query($query, $db_link);

		$number_locked = mysql_affected_rows( $db_link);


		if ($number_locked > 0) {

			// get the 'locked' rows ...

			$query = "SELECT * FROM audioblog_queue WHERE status = '%s'";
			$query = format_query( $query, $uniqueid);

			$db_result = mysql_query( $query, $db_link);

			if (!$db_result) {

				audioblog_error( "Error selecting locked audioblog entries: " . mysql_error());
				return -1;
			}


			// ... and try to upload them to the server

			while ($entry = mysql_fetch_assoc($db_result)) {

				$r = upload_audioblog_entry($entry);

				if ($r) {
					$entry['status'] = AB_UPLOADED;
					$entry['notes'] = "Node id of new entry: $r";

					$uploaded_entries_count++;

				} 
				else {
					// In case of error, add error message as a note in the associated database entry

					$entry['status'] = AB_ERROR;
					$entry['notes'] = "Error uploading entry to server: " . audioblog_error_msg();
					audioblog_log("Error uploading entry to server: " . audioblog_error_msg());
				}

				$entries_processed[] = $entry;
			}


			// now update the status of the processed entries on the database
				
			if (is_array( $entries_processed)) {

				foreach ($entries_processed as $entry) {
					
					$query = "UPDATE audioblog_queue SET status = '%s', notes = '%s' WHERE entry_id = %d";
					$query = format_query($query, $entry['status'], $entry['notes'], $entry['entry_id']);
					$db_result = mysql_query($query, $db_link);

					if (!$db_result) {
						
						audioblog_error("Error updating database: " . mysql_error());
						return -1;

					}
				
				}
			}
		}

	}


	// return the number of entries uploaded to the server

	audioblog_log("Uploaded $uploaded_entries_count entries to the server");

	return $uploaded_entries_count;
}


// -----------------------------------------------------------------------------
// private function definitions
// -----------------------------------------------------------------------------


/*
 * Upload the specified audioblog entry to the server.
 * 
 * @param $entry
 *   array with all the necessary parameters for the uploading
 * @return
 *   integer the id of the newly created blog entry (nid).  In case of error, 
 *          returns FALSE and stores the error description on audioblog_error_msg().
 */
function upload_audioblog_entry($entry) {


	// prepare file to be uploaded

        $fullname = $entry['file_path'] . DIRECTORY_SEPARATOR . $entry['file_name'];
        $fh = fopen($fullname, 'rb');

        if (!$fh) {

                audioblog_error( "Error opening file: $fullname");
                return FALSE;
        }

        $file_info['bits'] = "";     // doing this just to prevent PHPAGI from emailing me a warning message after the 'while' below...

        while (!feof($fh)) {
                $file_info['bits'] .= fread( $fh, 8192);
        }

        $file_info['bits'] = xmlrpc_base64($file_info['bits']);

        $file_info['name'] = $entry['file_name'];

	$file_info['mime_type'] = 'audio/x-wav';


	// prepare the content of the blog to be processed by the server

        $content['title'] = $entry['title'];
        $content['description'] = $entry['body'];
	$content['file_url_tmp'] = $entry['file_url_tmp'];
  

	// create a new entry with the uploaded information

        $rc = xmlrpc($entry['xmlrpc_url'],
                      'Audioblog.newPost', // 'metaWeblog.newPost',
                      $entry['blog_id'], // 'blog',//$blogs['blogid'],
                      $entry['drupal_uid'],
                      $entry['phone_pin'],
                      $content,
		      $file_info,
		      $entry['publish']
                     );

       if ($rc) {       // if xmlrpc call successful...

                if (!$rc->is_error) {

			// delete the local copy of the uploaded file

			$file_removed = unlink($fullname);

			if ($file_removed) {

                          return $rc;     // return the node id of the new blog entry
			}
			else {
				audioblog_error("Error removing uploaded file");
                        	return FALSE;
			}
                }
                else {

                        audioblog_error("Server error creating blog entry: $rc->message");
                        return FALSE;

                }

        }
        else {  // if xmlrpc call failed...

echo "\nThis is rc: $rc\n";
                audioblog_error("XMLRPC error creating audioblog entry: " . xmlrpc_error_msg());
                return FALSE;
        }


	// return the node id of the newly created audioblog entry

	return $rc;

}


/*
 * Create a formatted post (title and body) using templates and input data
 * 
 * @param
 *   object input data
 * @return
 *   object formatted post (title and body)
 */
function render_output($template_file, $replacements) {
 
	extract($replacements);        // Extract the vars to local namespace
	ob_start();                    // Start output buffering
	include($template_file);       // Include the file
	$contents = ob_get_contents(); // Get the contents of the buffer
	ob_end_clean();                // End buffering and discard
	return $contents;              // Return the contents
}  

/*
 * Formats the arguments for a database query
 * Note: this function is based on Drupal's db_query function
 * 
 * @param
 *   string query A string containing an SQL query.
 * @param
 *   ... A variable number of arguments which are substituted 
 *       into the query using printf() syntax. Instead of a 
 *       variable number of query arguments, you may also pass
 *       a single array containing the query arguments.
 * @return
 *   string with the new query arguments formatted
 */

function format_query($query) {
	$args = func_get_args();
	if (count($args) > 1) {
		if (is_array($args[1])) {
			$args = array_merge(array($query), $args[1]);
		}                
		$args = array_map('db_escape_string', $args);
		$args[0] = $query;
		$query = call_user_func_array('sprintf', $args);
	}                              
	return $query;        
}                                   

/*
 * Callback function called by format_query. 
 *    It inserts backslashes in the input argument in such a way
 *    that it becomes appropriate to be used in a database query.
 * 
 * @param
 *   string text to be modified
 * @return
 *   string the modified text
 */
function db_escape_string($text) {
	return addslashes($text);
}  




/*
 * Handles audioblog error messages.
 *
 * @param
 *   string error message
 * @return
 *   struct audioblog error
 */
function audioblog_error($message = NULL) {

	static $audioblog_error;

	if ($message) {
		$audioblog_error = new stdClass();
		$audioblog_error->is_error = TRUE;
		$audioblog_error->message = $message;
		audioblog_log($message);
	}

	return $audioblog_error;
}



/*
 * Returns the last audioblog error message.
 *
 * @return
 *   string audioblog error message
 */
function audioblog_error_msg() {
	$error = audioblog_error();
	return $error->message;
}



/*
 * Save the given line in the log file for debugging
 *
 * @param
 *   string text to be logged
 * @return
 *   void
 */
function audioblog_log($line) {
	global $audioblog_agi_debug_on;

	$log_file = fopen(AB_LOG_FILE, 'a');
	$current_time = date("Y.m.d-H:i:s");
	fputs($log_file, "[$current_time] " . $line . "\n");
	fclose($log_file);

        if ($audioblog_agi_debug_on) {
           echo "VERBOSE \"$line\" 2 \n";
        }
}


?>
